`default_nettype none
  
module admp421_read
  (
   input wire       CLK, RST, // 100MHz
   output reg       ADMP421_CLK, 
   output wire      ADMP421_LR,
   input wire       ADMP421_DATA,

   output reg [7:0] Q,
   output wire STROBE
   );

   parameter Period = 40; // 2.5MHz
   
   
   assign ADMP421_LR = 0;

   // 2.5MHz clock generator
   reg [7:0]   CLK_CNT;
   wire        CLK_CNT_FULL = (CLK_CNT == (Period/2-1));
   
   always @ (posedge CLK) begin
      if (RST) begin
         CLK_CNT <= 0;
         ADMP421_CLK <= 0;
      end else begin
         CLK_CNT <= CLK_CNT_FULL ? 0 : CLK_CNT + 1;
         ADMP421_CLK <= CLK_CNT_FULL ? ~ADMP421_CLK : ADMP421_CLK;
      end
   end

   // 128 samples counter
   reg CLK_R;
   wire CLK_POS = ~CLK_R & ADMP421_CLK;

   always @ (posedge CLK) CLK_R <= ADMP421_CLK;

   reg [6:0] SAMPLE_PERIOD_CNT;
   reg [7:0] SAMPLE_CNT1, SAMPLE_CNT2;

   wire      CNT1_TRIG = &SAMPLE_PERIOD_CNT;
   wire      CNT2_TRIG = (SAMPLE_PERIOD_CNT == 63);
     
   
   always @ (posedge CLK) begin
      if (RST) begin
         SAMPLE_PERIOD_CNT <= 0;
         SAMPLE_CNT1 <= 0;
      end else begin
         if (CLK_POS) begin
            SAMPLE_PERIOD_CNT <= SAMPLE_PERIOD_CNT+1;

            if (CNT1_TRIG) SAMPLE_CNT1 <= 0;
            else           if (ADMP421_DATA) SAMPLE_CNT1 <= SAMPLE_CNT1 + 1;

            if (CNT2_TRIG) SAMPLE_CNT2 <= 0;
            else           if (ADMP421_DATA) SAMPLE_CNT2 <= SAMPLE_CNT2 + 1;

            Q <= CNT1_TRIG ? SAMPLE_CNT1 :
                 CNT2_TRIG ? SAMPLE_CNT2 : Q;

         end
      end
   end // always @ (posedge CLK)

   wire STROBE_TMP = CLK_POS & (CNT1_TRIG | CNT2_TRIG);
   reg  STROBE_TMP_R;

   always @ (posedge CLK) 
      STROBE_TMP_R <= STROBE_TMP;
   
   assign STROBE = STROBE_TMP_R & ~STROBE_TMP;
   
   
endmodule

`default_nettype wire
  
