   
module powermeter
  (
   input wire         CLK, RST,
   input wire [7:0]   D,
   input wire         D_STROBE,
   output wire [15:0] LED
   );
   
   // LED turns off every 0.1sec @ 100MHz
   parameter TurnOffPeriod = 10 * 1000 * 1000;
   reg [27:0]         TURNOFF_TIMER;
   wire               TURNOFF;

   always @ (posedge CLK) 
      TURNOFF_TIMER <= ( RST | TURNOFF ) ? 0 :
                       TURNOFF_TIMER + 1;
   
   assign TURNOFF = (TURNOFF_TIMER == (TurnOffPeriod-1));

   wire [15:0]        LEVEL;

   assign LEVEL = ( ( D[7:4] == 4'hf ) ? 16'b1111_1111_1111_1111 : 
                    ( D[7:4] == 4'he ) ? 16'b1111_1111_1111_1110 : 
                    ( D[7:4] == 4'hd ) ? 16'b1111_1111_1111_1100 : 
                    ( D[7:4] == 4'hc ) ? 16'b1111_1111_1111_1000 : 
                    ( D[7:4] == 4'hb ) ? 16'b1111_1111_1111_0000 : 
                    ( D[7:4] == 4'ha ) ? 16'b1111_1111_1110_0000 : 
                    ( D[7:4] == 4'h9 ) ? 16'b1111_1111_1100_0000 : 
                    ( D[7:4] == 4'h8 ) ? 16'b1111_1111_1000_0000 : 
                    ( D[7:4] == 4'h7 ) ? 16'b1111_1111_0000_0000 : 
                    ( D[7:4] == 4'h6 ) ? 16'b1111_1110_0000_0000 : 
                    ( D[7:4] == 4'h5 ) ? 16'b1111_1100_0000_0000 : 
                    ( D[7:4] == 4'h4 ) ? 16'b1111_1000_0000_0000 : 
                    ( D[7:4] == 4'h3 ) ? 16'b1111_0000_0000_0000 : 
                    ( D[7:4] == 4'h2 ) ? 16'b1110_0000_0000_0000 : 
                    ( D[7:4] == 4'h1 ) ? 16'b1100_0000_0000_0000 : 
                    16'b1000_0000_0000_0000 ) ;

   reg [15:0]         POWER;
      
   always @ (posedge CLK) begin
      if (RST) begin
         POWER <= 0;
      end else begin
         if (D_STROBE) begin
            POWER <= POWER | LEVEL;
         end else begin
            if (TURNOFF) POWER <= { POWER[14:0], 1'b0 };
         end
      end
      
   end
   
   assign LED = POWER;
   
   
endmodule
