module ps2mouse
  ( input         CLK, RST,
    input 	  PS2_CLK, PS2_DAT,
    output [15:0] X, Y,
    output reg 	  LEFT, RIGHT, 
    output 	  STROBE );

   reg [3:0] 	  STAT;


   reg [7:0] 	  MOUSE_STAT, MOUSE_X, MOUSE_Y;
   wire 	  PS2_STROBE;
   wire [7:0] 	  PS2_BYTE;
	  
   ps2recv ps2r ( .CLK(CLK), .RST(RST), .PS2_CLK(PS2_CLK), .PS2_DAT(PS2_DAT),
		  .STROBE(PS2_STROBE), .KEYCODE(PS2_BYTE) );
   
   reg [15:0] 	  XPOS, YPOS;

   wire 	  STAT_LEFT =   MOUSE_STAT[0];
   wire 	  STAT_RIGHT =  MOUSE_STAT[1];
   wire 	  STAT_X_SIGN = MOUSE_STAT[4];
   wire 	  STAT_Y_SIGN = MOUSE_STAT[5];
   
   always @ (posedge CLK) begin
      if (RST) begin
	 STAT <= 4'b1000;

	 XPOS <= 0;
	 YPOS <= 0;
      end else begin
	 if (PS2_STROBE) begin 
	    STAT <= { STAT[0], STAT[3:1] };

	    case (STAT)
	      4'b1000:
		MOUSE_STAT <= PS2_BYTE;
	      4'b0100:
		MOUSE_X <= PS2_BYTE;
	      4'b0010:
		MOUSE_Y <= PS2_BYTE;
	      4'b0001: begin // we're done here!
		 LEFT  <= STAT_LEFT;
		 RIGHT <= STAT_RIGHT;

		 XPOS <= ( ~STAT_X_SIGN ? (XPOS + {8'b0, MOUSE_X}) :
			   (XPOS - { 8'b0, (8'hff-MOUSE_X) } ) );
		 YPOS <= ( ~STAT_Y_SIGN ? (YPOS + {8'b0, MOUSE_Y}) :
			   (YPOS - { 8'b0, (8'hff-MOUSE_Y) } ) );
		 
	      end
	    endcase
	 end
      end
   end

   assign X = XPOS;
   assign Y = YPOS;
   
   assign STROBE = PS2_STROBE & STAT[0];

endmodule // ps2mouse
