// PS/2 receiver

module ps2recv
  ( input CLK, RST, 
    output 	 BUSY, STROBE, 
    input 	 PS2_CLK, PS2_DAT, 
    output [7:0] KEYCODE );
   
   reg [7:0] 	PBUF, PBUF_R;
   reg 		STROBE;
   reg 		PS2_CLK_R1, PS2_CLK_R2, PS2_DATA_R;
   reg [10:0] 	PS2STAT;
   wire [7:0] 	KEYCODE;
   
   wire 	PS2EDGE;
   
   assign KEYCODE = PBUF_R;
   assign PS2EDGE = PS2_CLK_R2 & ~PS2_CLK_R1;
   assign BUSY = ~PS2STAT[0];

   always @ (posedge CLK) begin
      PS2_CLK_R1 <= PS2_CLK;
      PS2_CLK_R2 <= PS2_CLK_R1;
      PS2_DATA_R <= PS2_DAT;

      if (RST) begin
	 PS2STAT <= 11'b1;
	 PBUF   <= 8'h0;
      end else begin

	 if( PS2EDGE ) begin
	    casex (PS2STAT)
	      11'b00_0000_0000_1: begin // Idle -> received start bit
		 PS2STAT <= { PS2STAT[9:0], PS2STAT[10] };
		 PBUF <= 0;
	      end

	      11'b00_xxxx_xxxx_0: begin
		 PS2STAT <= { PS2STAT[9:0], PS2STAT[10] };
		 PBUF <= PBUF | ( PS2_DAT ? PS2STAT[8:1] : 8'b0 );
	      end
		
	      11'bxx_0000_0000_0: begin // parity & stop
		 PS2STAT <= { PS2STAT[9:0], PS2STAT[10] };
	      end

	      default:
		PS2STAT <= 11'b1;
	    endcase
	 end

	 if ( PS2EDGE && PS2STAT[10] && PS2_DAT ) begin
	    PBUF_R <= PBUF;
	    STROBE <= 1'b1;
	 end else begin
	    STROBE <= 1'b0;
	 end
      end
   end

endmodule // ps2_receiver
